@@ -1,17 +1,17 @@
 ## Fit conditional density model
 sscden1 <- function(formula,response,type=NULL,data=list(),weights,
                     subset,na.action=na.omit,alpha=1.4,
-                    id.basis=NULL,nbasis=NULL,seed=NULL,
+                    id.basis=NULL,nbasis=NULL,seed=NULL,rho=list("xy"),
                     ydomain=as.list(NULL),yquad=NULL,
                     prec=1e-7,maxiter=30,skip.iter=FALSE)
 {
     ## Obtain model frame and model terms
     mf <- match.call()
     mf$response <- mf$type <- mf$alpha <- NULL
-    mf$id.basis <- mf$nbasis <- mf$seed <- NULL
+    mf$id.basis <- mf$nbasis <- mf$seed <- mf$rho <- NULL
     mf$ydomain <- mf$yquad <- NULL
     mf$prec <- mf$maxiter <- mf$skip.iter <- NULL
-    term.wk <- terms.formula(mf$formula)
+    term.wk <- terms.formula(formula)
     ynames <- as.character(attr(terms(response),"variables"))[-1]
     mf[[1]] <- as.name("model.frame")
     mf <- eval(mf,parent.frame())
@@ -41,17 +41,27 @@ sscden1 <- function(formula,response,type=NULL,data=list(),weights,
     xnames <- vars[!(vars%in%ynames)]
     if (is.null(xnames)) stop("gss error in sscden1: missing covariate")
     ## Set type for given ydomain
+    fac.list <- NULL
     for (ylab in ynames) {
         y <- mf[[ylab]]
-        if (!is.factor(y)) {
-            if (!is.null(ydomain[[ylab]])) {
-                if (!is.vector(y)) stop("gss error in sscden: can only set domain in 1-D")
-                if (is.null(type[[ylab]]))
-                    type[[ylab]] <- list("cubic",ydomain[[ylab]])
-                else {
-                    if (length(type[[ylab]])==1)
-                        type[[ylab]] <- list(type[[ylab]][[1]],ydomain[[ylab]])
-                }
+        if (is.factor(y)) {
+            fac.list <- c(fac.list,ylab)
+            ydomain[[ylab]] <- NULL
+        }
+        else {
+            if (!is.vector(y))
+                stop("gss error in sscden1: no default quadrature")
+            if (is.null(ydomain[[ylab]])) {
+                mn <- min(y)
+                mx <- max(y)
+                ydomain[[ylab]] <- c(mn,mx)+c(-1,1)*(mx-mn)*.05
+            }
+            else ydomain[[ylab]] <- c(min(ydomain[[ylab]]),max(ydomain[[ylab]]))
+            if (is.null(type[[ylab]]))
+                type[[ylab]] <- list("cubic",ydomain[[ylab]])
+            else {
+                if (length(type[[ylab]])==1)
+                    type[[ylab]] <- list(type[[ylab]][[1]],ydomain[[ylab]])
             }
         }
     }
@@ -67,29 +77,138 @@ sscden1 <- function(formula,response,type=NULL,data=list(),weights,
     xx.wt <- xx.wt/sum(xx.wt)
     nx <- length(xx.wt)
     ## calculate rho
-    yfac <- TRUE
-    for (ylab in ynames) yfac <- yfac&is.factor(mf[,ylab])
-    if (!yfac) {
-        rho <- ssden(response,id.basis=id.basis,data=data,weights=cnt,
-                     alpha=2,domain=ydomain,quad=yquad)
-        rho.pt <- rho$quad$pt
-        rho.wt <- dssden(rho,rho.pt)*rho$quad$wt
-    }
-    else {
-        rho.pt <- data.frame(levels(mf[,ynames[1]]))
-        if (length(ynames)>1) {
-            for (ylab in ynames[-1]) {
-                wk <- expand.grid(levels(mf[,ylab]),1:dim(rho.pt)[1])
-                rho.pt <- data.frame(rho.pt[wk[,2],],wk[,1])
+    if (is.null(rho$fun)) {
+        type <- rho[[1]]
+        if (type=="y") {
+            yfac <- TRUE
+            for (ylab in ynames) yfac <- yfac&is.factor(mf[,ylab])
+            if (!yfac) {
+                if (is.null(cnt)) cntt <- rep(1,dim(mf)[1])
+                rho <- ssden(response,data=data,weights=cnt,id.basis=id.basis,
+                             alpha=2,domain=ydomain,quad=yquad)
+                qd.pt <- rho$quad$pt
+                qd.wt <- rho$quad$wt
+                env <- list(ydomain=ydomain,qd.pt=qd.pt,qd.wt=qd.wt,rho=rho)
+                fun <- function(x,y,env,outer.prod=FALSE) {
+                    if (!outer.prod) dssden(env$rho,y)
+                    else t(matrix(dssden(env$rho,y),dim(y)[1],dim(x)[1]))
+                }
             }
+            else {
+                qd.pt <- data.frame(levels(mf[,ynames[1]]))
+                if (length(ynames)>1) {
+                    for (ylab in ynames[-1]) {
+                        wk <- expand.grid(levels(mf[,ylab]),1:dim(qd.pt)[1])
+                        qd.pt <- data.frame(qd.pt[wk[,2],],wk[,1])
+                    }
+                }
+                colnames(qd.pt) <- ynames
+                qd.wt <- as.vector(table(mf[,rev(ynames)]))
+                qd.wt <- qd.wt/sum(qd.wt)
+                env <- list(qd.pt=qd.pt,qd.wt=qd.wt)
+                fun <- function(x,y,env,outer.prod=FALSE) {
+                    if (!outer.prod) rep(1,dim(x)[1])
+                    else matrix(1,dim(x)[1],dim(y)[1])
+                }
+            }
+            rho <- list(fun=fun,env=env)
+        }
+        if (type=="xy") {
+            ydomain <- data.frame(ydomain)
+            mn <- ydomain[1,]
+            mx <- ydomain[2,]
+            dm <- ncol(ydomain)
+            if (dm==1) {
+                ## Gauss-Legendre quadrature
+                quad <- gauss.quad(200,c(mn,mx))
+                quad$pt <- data.frame(quad$pt)
+                colnames(quad$pt) <- colnames(ydomain)
+            }
+            else {
+                ## Smolyak cubature
+                qdsz.depth <- switch(min(dm,6)-1,18,14,10,9,7)
+                quad <- smolyak.quad(dm,qdsz.depth)
+                for (i in 1:ncol(ydomain)) {
+                    ylab <- colnames(ydomain)[i]
+                    wk <- mf[[ylab]]
+                    jk <- ssden(~wk,domain=data.frame(wk=ydomain[,i]),alpha=2,
+                                id.basis=id.basis,weights=cnt)
+                    quad$pt[,i] <- qssden(jk,quad$pt[,i])
+                    quad$wt <- quad$wt/dssden(jk,quad$pt[,i])
+                }
+                jk <- wk <- NULL
+                quad$pt <- data.frame(quad$pt)
+                colnames(quad$pt) <- colnames(ydomain)
+            }
+            ## Incorporate factors in quadrature
+            if (!is.null(fac.list)) {
+                for (i in 1:length(fac.list)) {
+                    wk <- expand.grid(levels(mf[[fac.list[i]]]),1:length(quad$wt))
+                    quad$wt <- quad$wt[wk[,2]]
+                    col.names <- c(fac.list[i],colnames(quad$pt))
+                    quad$pt <- data.frame(wk[,1],quad$pt[wk[,2],])
+                    colnames(quad$pt) <- col.names
+                }
+            }
+            rho <- list(NULL)
+            for (ylab in ynames) {
+                if (is.numeric(mf[[ylab]])) {
+                    form <- as.formula(paste(ylab,"~",paste(xnames,collapse="+")))
+                    rho[[ylab]] <- ssanova(form,data=mf,id.basis=id.basis)
+                }
+                if (is.factor(mf[[ylab]])) {
+                    form <- as.formula(paste("~(",paste(xnames,collapse="+"),")*",ylab))
+                    resp <- as.formula(paste("~",ylab))
+                    rho[[ylab]] <- ssllrm(form,resp,data=mf,id.basis=id.basis)
+                }
+            }
+            env <- list(ynames=ynames,ydomain=ydomain,qd.pt=quad$pt,qd.wt=quad$wt,rho=rho)
+            fun <- function(x,y,env,outer.prod=FALSE) {
+                z <- 1
+                for (ylab in env$ynames) {
+                    yy <- y[[ylab]]
+                    if (is.numeric(yy)) {
+                        mu <- predict(env$rho[[ylab]],x)
+                        sigma <- sqrt(env$rho[[ylab]]$varht)
+                        ymn <- env$ydomain[1,ylab]
+                        ymx <- env$ydomain[2,ylab]
+                        if (!outer.prod) {
+                            wk <- dnorm((yy-mu)/sigma)/
+                                (pnorm((ymx-mu)/sigma)-pnorm((ymn-mu)/sigma))
+                            z <- z*wk
+                        }
+                        else {
+                            wk <- t(outer(yy,mu,dnorm,sigma))/
+                                (pnorm((ymx-mu)/sigma)-pnorm((ymn-mu)/sigma))
+                            z <- z*wk
+                        }
+                    }
+                    if (is.factor(yy)) {
+                        wk <- predict(env$rho[[ylab]],x)
+                        if (!outer.prod) {
+                            wk1 <- NULL
+                            for (i in 1:length(yy))
+                                wk1 <- c(wk1,wk[i,yy[i]==env$rho[[ylab]]$qd.pt])
+                            z <- z*wk1
+                        }
+                        else {
+                            wk1 <- NULL
+                            for (i in 1:length(yy))
+                                wk1 <- cbind(wk1,wk[,yy[i]==env$rho[[ylab]]$qd.pt])
+                            z <- z*wk1
+                        }
+                    }
+                }
+                z
+            }
+            rho <- list(fun=fun,env=env)
         }
-        colnames(rho.pt) <- ynames
-        rho.wt <- as.vector(table(mf[,rev(ynames)]))
-        rho.wt <- rho.wt/sum(rho.wt)
-        rho <- list(pt=rho.pt,wt=rho.wt)
     }
     ## Generate s, r, int.s, and int.r
-    nmesh <- length(rho.wt)
+    rho.wk <- rho$fun(x[!x.dup.ind,,drop=FALSE],rho$env$qd.pt,rho$env,outer=TRUE)
+    rho.wk <- t(t(rho.wk)*rho$env$qd.wt)
+    rho.wk1 <- apply(rho.wk*xx.wt,2,sum)
+    nmesh <- length(rho$env$qd.wt)
     s <- r <- int.s <- int.r <- NULL
     id.s <- id.r <- NULL
     id.s.list <- id.r.list <- list(NULL)
@@ -102,7 +221,7 @@ sscden1 <- function(formula,response,type=NULL,data=list(),weights,
         xy.basis <- mf[id.basis,vlist]
         qd.xy <- data.frame(matrix(0,nmesh,length(vlist)))
         names(qd.xy) <- vlist
-        qd.xy[,y.list] <- rho.pt[,y.list]
+        qd.xy[,y.list] <- rho$env$qd.pt[,y.list]
         if (length(x.list)) xx <- x[!x.dup.ind,x.list,drop=FALSE]
         else xx <- NULL
         nphi <- term[[label]]$nphi
@@ -118,7 +237,7 @@ sscden1 <- function(formula,response,type=NULL,data=list(),weights,
                     id.s <- c(id.s,nu)
                     id.s.list[[label]] <- c(id.s.list[[label]],nu)
                     qd.s.wk <- phi$fun(qd.xy[,,drop=TRUE],nu=i,env=phi$env)
-                    int.s <- c(int.s,sum(qd.s.wk*rho.wt))
+                    int.s <- c(int.s,sum(qd.s.wk*rho.wk1))
                 }
                 else {
                     if (length(y.list)==0) {
@@ -132,7 +251,7 @@ sscden1 <- function(formula,response,type=NULL,data=list(),weights,
                         for (j in 1:nx) {
                             qd.xy[,x.list] <- xx[rep(j,nmesh),]
                             qd.s.wk <- phi$fun(qd.xy,i,phi$env)
-                            int.s.wk <- int.s.wk + sum(qd.s.wk*rho.wt)*xx.wt[j]
+                            int.s.wk <- int.s.wk + sum(qd.s.wk*rho.wk[j,])*xx.wt[j]
                         }
                         int.s <- c(int.s,int.s.wk)
                     }
@@ -150,7 +269,7 @@ sscden1 <- function(formula,response,type=NULL,data=list(),weights,
                     id.r <- c(id.r,nq)
                     id.r.list[[label]] <- c(id.r.list[[label]],nq)
                     qd.r.wk <- rk$fun(qd.xy[,,drop=TRUE],xy.basis,nu=i,env=rk$env,out=TRUE)
-                    int.r <- cbind(int.r,apply(rho.wt*qd.r.wk,2,sum))
+                    int.r <- cbind(int.r,apply(rho.wk1*qd.r.wk,2,sum))
                 }
                 else {
                     if (length(y.list)==0) {
@@ -165,7 +284,7 @@ sscden1 <- function(formula,response,type=NULL,data=list(),weights,
                         for (j in 1:nx) {
                             qd.xy[,x.list] <- xx[rep(j,nmesh),]
                             qd.r.wk <- rk$fun(qd.xy,xy.basis,i,rk$env,TRUE)
-                            int.r.wk <- int.r.wk + apply(rho.wt*qd.r.wk,2,sum)*xx.wt[j]
+                            int.r.wk <- int.r.wk + apply(rho.wk[j,]*qd.r.wk,2,sum)*xx.wt[j]
                         }
                         int.r <- cbind(int.r,int.r.wk)
                     }
